﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using com.citizen.sdk;

namespace DeviceControlSample2
{
    public partial class Form1 : Form
    {
        private ESCPOSPrinter printer;
        private LineDisplay display;
        private Scanner scanner;

        delegate void SetStatusCallback(string text, Color bgcolor);
        delegate void SetDataCallback(string text);

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            DataTable dataTable1 = new DataTable("ComboBox");
            dataTable1.Columns.Add("Display", typeof(string));
            dataTable1.Columns.Add("Value", typeof(int));
            dataTable1.Rows.Add("WiFi", ESCPOSConst.CMP_PORT_WiFi);
            //dataTable1.Rows.Add("Bluetooth", ESCPOSConst.CMP_PORT_Bluetooth);
            comboBoxType.DataSource = dataTable1;
            comboBoxType.DisplayMember = "Display";
            comboBoxType.ValueMember = "Value";
            comboBoxType.SelectedIndex = 0;

            TxtAddress1.Text = "192.168.234.100";
            TxtAddress2.Text = "192.168.234.100";
            TxtAddress3.Text = "192.168.234.100";
            TxtPort1.Text = "9100";
            TxtPort2.Text = "9200";
            TxtPort3.Text = "9210";
            TxtStatus.Text = "Stopped";
            TxtStatus.BackColor = Color.LightGray;

            printer = new ESCPOSPrinter();
            display = new LineDisplay();
            scanner = new Scanner();

            // Set Scanner events.
            scanner.StatusUpdateEvent += new StatusUpdateEventHandler(OnStatusUpdateEvent);
            scanner.DataEvent += new DataEventHandler(OnDataEvent);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            printer.Dispose();
            display.Dispose();
            scanner.Dispose();
        }

        private void BtnSearch_Click(object sender, EventArgs e)
        {
            int errCode;
            CitizenPrinterInfo[] list;
            int type = (int)comboBoxType.SelectedValue;

            Cursor preCursor = Cursor.Current;
            Cursor.Current = Cursors.WaitCursor;

            list = printer.SearchCitizenPrinter(type, 5, out errCode);

            Cursor.Current = preCursor;

            if (errCode != ESCPOSConst.CMP_SUCCESS)
            {
                MessageBox.Show("Printer not found.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            fSearch frmSearch = new fSearch();
            frmSearch.PrinterList = list;
            frmSearch.ShowDialog();
            if (frmSearch.SelectedAddress != "")
            {
                TxtAddress1.Text = frmSearch.SelectedAddress;
                TxtAddress2.Text = frmSearch.SelectedAddress;
                TxtAddress3.Text = frmSearch.SelectedAddress;
            }
        }

        private void BtnPrinterConnect_Click(object sender, EventArgs e)
        {
            int result;
            int type = (int)comboBoxType.SelectedValue;
            string addr = TxtAddress1.Text;
            int port;

            try
            {
                port = Int32.Parse(TxtPort1.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("Port number invalid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            result = printer.Connect(type, addr, port);
            if (result == ESCPOSConst.CMP_SUCCESS)
            {
                MessageBox.Show("Printer Connect Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Printer Connect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnPrinterDisconnect_Click(object sender, EventArgs e)
        {
            int result = printer.Disconnect();
            if (result == ESCPOSConst.CMP_SUCCESS)
            {
                MessageBox.Show("Printer Disconnect Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Printer Disconnect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnPrinterStatus_Click(object sender, EventArgs e)
        {
            // Printer Check
            int result = printer.PrinterCheck();
            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                String msg = "PrinterCheck() : Success\n";
                
                // Get Status
                int status = printer.Status();
                msg += "\nStatus() : ";
                
                // Status Check
                if (ESCPOSConst.CMP_STS_NORMAL == status)
                {
                    // No Error
                    msg += "\n\t * Normal ( No Error )";
                }
                else
                {
                    if ((ESCPOSConst.CMP_STS_COVER_OPEN & status) > 0)
                    {
                        // Cover Open
                        msg += "\n\t * Cover Open";
                    }
                    if ((ESCPOSConst.CMP_STS_PAPER_EMPTY & status) > 0)
                    {
                        // Paper Empty
                        msg += "\n\t * Paper Empty";
                    }
                    if ((ESCPOSConst.CMP_STS_PRINTEROFF & status) > 0)
                    {
                        // Printer Offline
                        msg += "\n\t * Printer Offline";
                    }
                }
                MessageBox.Show(msg, "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                // PrinterCheck Error
                MessageBox.Show("PrinterCheck() Error : " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnSamplePrint_Click(object sender, EventArgs e)
        {
            printer.SetEncoding("UTF-8");

            printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

            printer.PrintTextPCFont("ABC Shop\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, "Arial", 25, ESCPOSConst.CMP_FNT_BOLD | ESCPOSConst.CMP_FNT_ITALIC, 100, 100);
            printer.PrintText("\nThank you very much.\nLooking forward to your visit again.\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_BOLD, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText(DateTime.Now.ToString() + "\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

            printer.PrintText("1001  Hamburger          4   12.00\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("1002  French fries       2    4.00\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("1003  Orange juice       2    3.00\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("1004  Apple juice        1    1.50\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("1005  Coke               2    2.00\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("\nSub Total  ", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("       22.50", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("\nTax        ", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("        1.80", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintText("\nTotal       24.30\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
            printer.PrintBarCode("100000000012", ESCPOSConst.CMP_BCS_JAN13, 80, 4, ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

            printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

            int result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

            if (result == ESCPOSConst.CMP_SUCCESS)
            {
                MessageBox.Show("Print Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Print Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void BtnBarcodePrint_Click(object sender, EventArgs e)
        {
            printer.TransactionPrint(ESCPOSConst.CMP_TP_TRANSACTION);

            printer.PrintText("[ UPC-A ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintBarCode("123456789012", ESCPOSConst.CMP_BCS_UPCA, 100, 3, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n\n[ JAN-13 ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintBarCode("123456789012", ESCPOSConst.CMP_BCS_JAN13, 100, 3, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n\n[ CODE 39 ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintBarCode("ABC123", ESCPOSConst.CMP_BCS_Code39, 100, 3, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n\n[ CODE 128 ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintBarCode("ABCD1234", ESCPOSConst.CMP_BCS_Code128, 100, 3, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n\n[ CODABAR ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintBarCode("A23456789A", ESCPOSConst.CMP_BCS_Codabar, 100, 3, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
            printer.PrintText("\n\n[ QR Code ]\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);
            printer.PrintQRCode("http://www.citizen-systems.co.jp/", 6, ESCPOSConst.CMP_QRCODE_EC_LEVEL_M, ESCPOSConst.CMP_ALIGNMENT_LEFT);
            printer.PrintText("\n\n\n", ESCPOSConst.CMP_ALIGNMENT_LEFT, 0, 0);

            printer.CutPaper(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);
            int result = printer.TransactionPrint(ESCPOSConst.CMP_TP_NORMAL);

            if (result == ESCPOSConst.CMP_SUCCESS)
            {
                MessageBox.Show("Print Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Print Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnDisplayConnect_Click(object sender, EventArgs e)
        {
            int result;
            int type = (int)comboBoxType.SelectedValue;
            string addr = TxtAddress2.Text;
            int port;

            try
            {
                port = Int32.Parse(TxtPort2.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("Port number invalid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            result = display.Connect(type, addr, port);
            if (result == LineDisplayConst.CDP_SUCCESS)
            {
                MessageBox.Show("LineDisplay Connect Success. ", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("LineDisplay Connect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnDisplayDisconnect_Click(object sender, EventArgs e)
        {
            int result = display.Disconnect();
            if (result == LineDisplayConst.CDP_SUCCESS)
            {
                MessageBox.Show("LineDisplay Disconnect Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("LineDisplay Disconnect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void BtnDisplayStatus_Click(object sender, EventArgs e)
        {
            // Display Check
            int result = display.DisplayCheck();
            if (LineDisplayConst.CDP_SUCCESS == result)
            {
                MessageBox.Show("DisplayCheck() : Success\n", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                // Display Check Error
                MessageBox.Show("DisplayCheck() Error : " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnSampleDisplay_Click(object sender, EventArgs e)
        {
            display.SetEncoding("Shift_JIS");
            display.ClearDisplay();
            display.DisplayText("Thank you so much!");
            display.SetCursorPosition(13, 2);
            int result = display.DisplayText("$299.99-", true);

            if (result == LineDisplayConst.CDP_SUCCESS)
            {
                MessageBox.Show("Display Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Display Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnClearDisplay_Click(object sender, EventArgs e)
        {
            int result = display.ClearDisplay();

            if (result == LineDisplayConst.CDP_SUCCESS)
            {
                MessageBox.Show("Display Success.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Display Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void BtnScannerConnect_Click(object sender, EventArgs e)
        {
            int result;
            int type = (int)comboBoxType.SelectedValue;
            string addr = TxtAddress3.Text;
            int port;

            try
            {
                port = Int32.Parse(TxtPort3.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("Port number invalid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            result = scanner.Connect(type, addr, port);
            if (result != ScannerConst.CSC_SUCCESS)
            {
                MessageBox.Show("Scanner Connect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            TxtScanData.Text = "";
        }

        private void BtnScannerDisconnect_Click(object sender, EventArgs e)
        {
            int result;

            result = scanner.Disconnect();
            if (result != ScannerConst.CSC_SUCCESS)
            {
                MessageBox.Show("Scanner Disconnect Error: " + result.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            TxtStatus.Text = "Stopped";
            TxtStatus.BackColor = Color.LightGray;
        }

        // Scanner StatusUpdate Event
        private void OnStatusUpdateEvent(int status)
        {
            Trace.WriteLine("Status: " + status);
            switch (status)
            {
                case ScannerConst.CSC_SUE_POWER_ONLINE:
                    this.ShowStatus("Online", Color.LightGreen);
                    break;
                case ScannerConst.CSC_SUE_POWER_OFF:
                    this.ShowStatus("Offline", Color.LightPink);
                    break;
                default:
                    break;
            }
        }

        private void ShowStatus(string text, Color bgcolor)
        {
            if (this.TxtStatus.InvokeRequired)
            {
                SetStatusCallback d = new SetStatusCallback(ShowStatus);
                this.Invoke(d, new object[] { text, bgcolor });
            }
            else
            {
                this.TxtStatus.Text = text;
                this.TxtStatus.BackColor = bgcolor;
            }
        }

        // Scanner Data Event
        private void OnDataEvent(byte[] data)
        {
            this.ShowData(Encoding.UTF8.GetString(data));
        }

        private void ShowData(string text)
        {
            if (this.TxtScanData.InvokeRequired)
            {
                SetDataCallback d = new SetDataCallback(ShowData);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                if (this.TxtScanData.Text.Length > 0) this.TxtScanData.AppendText("\r\n");
                this.TxtScanData.AppendText(text);
            }
        }
    }
}
